package core.users;

import core.entities.User;
import core.repositories.UserRepository;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.Optional;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.junit.jupiter.api.Assertions.*;

@DataJpaTest
@DirtiesContext
@ExtendWith(SpringExtension.class)
public class UserRepositoryTests {

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private TestEntityManager testEntityManager;

    @Test
    void test_findByUsername() {
        //Situation 1: No user is present in the database, so no user should be found by a given username
        assertTrue(userRepository.findByUsername("FawKes100").isEmpty());

        //persist a user to the database, so a user is present
        testEntityManager.persist(new User("FawKes100", "mail@fawkes100.de", "123456"));

        //Situation 2: User with name "FawKes100" is present
        Optional<User> findResult = userRepository.findByUsername("FawKes100");
        assertTrue(findResult.isPresent());
        assertEquals(new User("FawKes100", "mail@fawkes100.de", "123456"),findResult.get());

        //Situation 3: No user with the required name is present
        findResult = userRepository.findByUsername("fawkes1001");
        assertFalse(findResult.isPresent());
    }

    @Test
    void test_existsByMail() {
        //Situation 1: invalid email format should not matter
        assertFalse(userRepository.existsByEmail("FawKes100"));

        //Situation 2: No user exists with such a mail
        assertFalse(userRepository.existsByEmail("mail@fawkes100.de"));

        //persist a user to the database, so a user is present
        testEntityManager.persist(new User("FawKes100", "mail@fawkes100.de", "123456"));

        //Situation 3: User with this mail exists
        assertTrue(userRepository.existsByEmail("mail@fawkes100.de"));

        //Situation 4: No user exists with such a mail (non-empty db)
        assertFalse(userRepository.existsByEmail("mail2@fawkes100.de"));
    }

    @Test
    void test_existsByUsername() {
        //Situation 1: Empty database
        assertFalse(userRepository.existsByUsername("FawKes100"));

        //persist a user to the database, so a user is present
        testEntityManager.persist(new User("FawKes100", "mail@fawkes100.de", "123456"));

        //Situation 3: User with this name exists
        assertTrue(userRepository.existsByUsername("FawKes100"));

        //Situation 4: No user exists with such a mail (non-empty db)
        assertFalse(userRepository.existsByUsername("FawKes101"));
    }

    @Test
    void test_deleteUserByName() {
        //persist a user to the database, so a user is present
        User user = testEntityManager.persist(new User("FawKes100", "mail@fawkes100.de", "123456"));
        User user1 = testEntityManager.persist(new User("FawKes101", "mail@fawkes101.de", "123456"));

        userRepository.deleteByUsername("FawKes100");

        assertThat(testEntityManager.find(User.class, user.getId())).isNull();
        assertThat(testEntityManager.find(User.class, user1.getId())).isNotNull();
    }


    @Test
    void test_countUsers() {
        assertEquals(0, userRepository.countUsers());

        testEntityManager.persist(new User("FawKes100", "mail@fawkes100.de", "123456"));
        assertEquals(1, userRepository.countUsers());

    }

}
